#version 430 core
/*
* Copyright 2018 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
} vert;

out vec4 color;

uniform vec3 camera_world;

uniform vec3 materialCoefficients; // x = ambient, y = diffuse, z = specular 
uniform float specularAlpha;
uniform sampler2D diffuseTexture;

const int levels = 5;
const float scaleFactor = 1.0 / levels;
int material_shininess = 1;

/*uniform struct PointLight {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointL;*/

uniform struct SpotLight {
	vec3 position;
	vec3 direction;
	float cutOff;
} spotL;

uniform float brightness = 0.0;

vec3 rgb2hsv(vec3 c)
{
    vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
    vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));

    float d = q.x - min(q.w, q.y);
    float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}

vec3 hsv2rgb(vec3 c)
{
    vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
    vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
    return c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y);
}

void main() {	
	vec3 n = normalize(vert.normal_world);
	vec3 L = normalize(spotL.position - vert.position_world);
	vec3 V = normalize(camera_world - vert.position_world);
	vec3 H = normalize(L + V);
	vec3 texColor = texture(diffuseTexture, vert.uv).rgb;
	float theta = dot(L, normalize(spotL.direction));

	if(theta>spotL.cutOff){
		float diffuse = max(0, dot(L,n));
		vec3 diffuseColor = texColor * materialCoefficients.y;
		vec3 diffuseColor_hsv = rgb2hsv(diffuseColor);
		diffuseColor_hsv.z = diffuseColor_hsv.z * floor(diffuse * levels) * scaleFactor;
		diffuseColor = hsv2rgb(diffuseColor_hsv);
		

		float specular = 0.0;
		
		if( dot(L,n) > 0.0)
		{
			specular = materialCoefficients.z * pow( max(0, dot( H, n)), material_shininess);
		}
		
		float edgeDetection = (dot(V, n) > 0.12) ? 1 : 0;
		float specMask = (pow(dot(H, n), material_shininess) > 0.4) ? 1 : 0;
		
		color = vec4(edgeDetection *(texColor * materialCoefficients.x + diffuseColor + specular * specMask)*brightness,1);

	}else{
		color = vec4((texColor * materialCoefficients.x*0.5)*brightness,1);
	}


}

